#!/usr/bin/python
import MySQLdb as mdb
import time

#
# Get all rows from MySQL (about 22 MB worth of data)
#
db = mdb.connect(host='localhost', user='MYSQL_USER', passwd='MYSQL_PASS', db='MYSQL_DB')
c = db.cursor()
c.execute("""SELECT * FROM stories ORDER BY snapshot_epoch ASC""")
rows = c.fetchall()
db.close()

fall_rate_data = {}
vote_rate_data = {}
last_seen = {}
upvoted_stories = {}

for row in rows:
  pos = row[0]
  href = row[1]
  submit_ep = int(row[4])
  snap_ep = int(row[5])
  score = row[6]
  submitter = row[7]

  # Convert timestamp to "day_of_week-hour_of_day"
  parsed_ts = time.localtime(snap_ep)
  hour_index = '%03d' % (parsed_ts.tm_wday*24 + parsed_ts.tm_hour)
  quantize = hour_index + ':' + time.strftime("%A-%H", time.localtime(snap_ep))
  if not (quantize in fall_rate_data):
    fall_rate_data[quantize] = []
    vote_rate_data[quantize] = []

  # Record increasing scores
  if not (href in upvoted_stories) and score > 1:
    quantize_submit = hour_index + ':' + time.strftime("%A-%H", time.localtime(submit_ep))
    upvoted_stories[href] = [ (score, snap_ep, submit_ep, quantize_submit) ]
  if href in upvoted_stories:
    if score > upvoted_stories[href][-1][0]:
      upvoted_stories[href].append( (score, snap_ep, submit_ep ) )

  if href in last_seen:
    (last_snapshot, last_pos, last_score, last_quantize) = last_seen[href]
    pos_delta = float(pos - last_pos)
    time_delta = float(snap_ep - last_snapshot)
    votes_since = score - last_score
    if votes_since < 0:
      # This is the dirty trick: delete the submission and then re-submit it, which 
      # causes votes_since to be negative. Since Hacker News does not have downvoting on
      # submissions, we know that this is some funny business.
      # print "DIRTY", votes_since, last_score, score, submitter
      continue
    if time_delta <= 0 or time_delta >= 800:
      # Data cleaning: this affects about 2% of data points.
      continue

    fall_rate = pos_delta / (time_delta / 60.0)
    vote_rate = votes_since / (time_delta / 60.0)

    fall_rate_data[quantize].append(fall_rate)
    if (votes_since > 0):
      vote_rate_data[quantize].append(vote_rate)

  # Record data
  last_seen[href] = (snap_ep, pos, score, quantize)

#
# Output files for plotting in R
#
f_fall_rate = open("fall-rate.txt", "w")
f_vote_rate = open("vote-rate.txt", "w")

f_fall_rate.write("epoch rate\n")
for ep in sorted(fall_rate_data.keys()):
  for dp in fall_rate_data[ep]:
    f_fall_rate.write(str(ep) + " " + str(dp) + "\n")
f_fall_rate.close()

f_vote_rate.write("epoch votes\n")
for ep in sorted(vote_rate_data.keys()):
  for dp in vote_rate_data[ep]:
    f_vote_rate.write(str(ep) + " " + str(dp) + "\n")
f_vote_rate.close()

#
# Output voting behavior for stories that got upvotes
#
f_upvoted = open("upvoted-stories.txt", "w")
f_upvoted.write("max_score time_to_first_upvote submit_time submit_quant\n")
for href in upvoted_stories:
  sequence = upvoted_stories[href]
  max_score = str(sequence[-1][0])
  submit_ep = str(sequence[-1][2])
  submit_quant = str(sequence[0][3])
  time_to_first_upvote = str(sequence[0][1] - sequence[0][2])
  f_upvoted.write(max_score+' '+time_to_first_upvote+' '+submit_ep+' '+submit_quant+'\n')
f_upvoted.close()
